# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action BaculaConfig
#
# Configure des sauvegardes bacula
#
###########################################################################

""" Actions permettant de gérer les sauvegardes bacula """
from twisted.python import log
from os.path import isfile

from creole import parsedico
from creole.eolelock import check_lock

from ead2.backend.config.config import BACULA_DIR
from ead2.backend.config.filenames import bacula as baculafiles
from ead2.backend.lib.action import Action, Dict
from ead2.backend.actions import tools
from ead2.backend.actions.tool.bacula import get_actual_support, \
        USED_TEMPLATES, FILENAMES
from ead2.backend.actions.lib.widgets import main as M, form as F, ajax

def test_mail(mail_list_str):
    """ teste une liste de mail pour la conf bacula
        mail,mail,... sans espace avec un arrobas par mail
    """
    if ' ' in mail_list_str:
        return False, "\\nErreur à la configuration de messagerie: ne pas mettre d'espace dans la liste des mails."
    mail_list = mail_list_str.split(',')
    for mail in mail_list:
        if not tools.test_mail(mail):
            return False, "\\nErreur à la configuration de messagerie: une adresse mail doit contenir un @ (et un seul)."
    return True, ''

class BaculaConfig(Action):
    """
    interface de configuration de l'outil de sauvegarde
    """
    user_description = Dict(default={},
                            doc="description de l'exécutant",
                            keys=['ip', 'name', 'role'])
    name = 'bacula_config'
    libelle = 'Configuration'
    category = 'Sauvegardes'
    description = "configuration de l'outil de sauvegarde bacula"
    request = Dict(default={},
                   doc="arguments de la requete en cours cote frontend",
                   keys=['server', 'action', 'save_type'])
    form_result = Dict(default={}, doc="retour de formulaire",
                       keys=['config1', 'config2', 'config3'])

    def execute(self):
        """ renvoie les donnees pour la mise en forme de l'action
            1 - Récupère le type de sauvegarde à configurer
            2 - Renvoie le formulaire de configuration bacula
            3 - Traite les retours formulaires et configure la sauvegarde
        """
        params, self.server_nb = tools.get_request(self.request)
        result = {'titre':self.description}
        ## 1 -
        if params.has_key('save_type'):
            self.save_type = params['save_type'][0]
        else:
            self.save_type = get_actual_support()
        ## 3 -
        if self.form_result != {} :
            try:
                datas = self.modify_conf()
                datas['toexec'] = tools.make_js_link(self.server_nb,
                                                     self.name, confirm=True)
            except Exception, mess:
                log.err()
                datas = dict(message=mess.message)
            self.form_result = {}
            return self.send_frag(datas, template='bacula_config',
                                  templates=USED_TEMPLATES)
        ## 2 -
        if self.save_type == '':
            # distant par défaut
            self.save_type = 'distant'
        result['formulaires'] = [ {'titre': 'Support de sauvegarde',
                                   'formulaire':self._get_main_form(),
                                   'name':'config1'},
                                  {'titre':'Paramètres de sauvegarde pour : %s' % self.save_type,
                                     'formulaire':self._get_form(),
                                     'name':'config2'},
                                  {'titre': "Test du montage",
                                     'formulaire':self._get_test_form(),
                                     'name':'test'},
                                  {'titre':"Paramètres d'envoi des logs (Facultatif)",
                                     'formulaire':self._get_log_form(),
                                     'name':'config3', 'validate':self._get_validate_btn()}]

        if self.save_type == 'bande':
            result['entete_bande'] = """<font color=red>La configuration est <b>manuelle</b>.
                                        Voir le fichier '/etc/bacula/bacula-sd.conf'</font>"""


        return self.send_all(result,
                             template='bacula_config',
                             templates=USED_TEMPLATES)

    def _get_main_form(self):
        """ renvoit le formulaire pour le choix du type de sauvegarde a configurer """
        tag = F.Select(name='support',
                       libelle='Support de sauvegarde',
                       inline=True,
                       onchange=ajax.call(self.server_nb,
                                          self.name,
                                          js_args=True,
                                          save_type='this.options[this.selectedIndex].value'))
        possibilities = {'distant':'Distant',
                         'usb':'Disque usb local',
                         'bande':'Bande: Configuration manuelle'}
        for name, libelle in possibilities.items():
            if name == self.save_type:
                tag.add_option(name, libelle=libelle, default=True)
            else:
                tag.add_option(name, libelle=libelle)
        tag['_type'] = 'choice'
        return [tag]

    def _get_form(self):
        """ renvoit le formulaire pour configurer les sauvegardes du type self.save_type """
        # configuration usb
        if self.save_type == 'usb':
            tag = F.Input(name='usb_periph',
                          libelle="Chemin d'accès",
                          inline=True,
                          required=True)
            if len(self._get_usb_conf()) > 0:
                tag.default(self._get_usb_conf()[0])
            tag['_type'] = 'text_input'
            return [tag]
        # configuration distante
        elif self.save_type == 'distant':
            tag1 = F.Input(name='machine',
                    libelle="Nom machine distante",
                    inline=True,
                    required=True)
            tag1['_type'] = 'text_input'
            tag2 = F.Input(name='ipmachine',
                           libelle="IP machine distante",
                           inline=True,
                           required=True)
            tag2['_type'] = 'text_input'
            tag3 = F.Input(name='partage',
                           libelle="Partage",
                           inline=True,
                           required=True)
            tag3['_type'] = 'text_input'
            tag4 = F.Input(name='login',
                           libelle="Login (facultatif)",
                           inline=True)
            tag4['_type'] = 'text_input'
            tag5 = F.Input(name='password',
                           libelle="Mot de passe (facultatif)",
                           inline=True)
            tag5['_type'] = 'password'
            conf = self._get_distant_conf()
            if len(conf)>0:
                tag1.default(conf[0])
                tag2.default(conf[1])
                tag3.default(conf[2])
                if len(conf) == 5:
                    tag4.default(conf[3])
                    tag5.default(conf[4])
            return [tag1, tag2, tag3, tag4, tag5]
        else:
            return []

    def _get_log_form(self):
        """ renvoie le formulaire pour la configuration des logs """
        mail, mailok, smtp = self._get_actual_log_conf()
        if not smtp:
            dic = parsedico.parse_dico()
            smtp = dic.get('passerelle_smtp', '')
            if not smtp:
                smtp = dic.get('passerelle_smtp_aca', '')
        tag1 = F.Input(name='mail',
                        libelle='Mail admin sauvegarde pour les erreurs',
                        inline=True)
        tag1.default(mail)
        tag1['_type'] = 'text_input'
        tag2 = F.Input(name='mailok',
                       libelle='Mail admin sauvegarde',
                       inline=True)
        tag2.default(mailok)
        tag2['_type'] = 'text_input'
        tag3 = F.Input(name='smtp',
                       libelle='Smtp',
                       inline=True)
        tag3.default(smtp)
        tag3['_type'] = 'text_input'
        return [tag1, tag2, tag3]

    def _get_test_form(self):
        """ renvoie la partie test de la configuration """
        tag = {'_type':'text'}
        if check_lock('eolesauvegarde'):
            diode = '/image/diode_unknown.png'
            retour = "Test impossible : une sauvegarde est en cours"
        else:
            code, retour = tools.command_statusoutput('%stest_montage.sh' % BACULA_DIR)
            if code == 0:
                diode = '/image/diode_on.png'
            else:
                diode = '/image/diode_off.png'
        tag['text'] = "<img src=\"%s\"/><br/>" % diode
        for line in retour.split('\n'):
            if line and not line.startswith('####'):
                tag['text'] += "%s<br/>" % line
        return [tag]

    def _get_distant_conf(self):
        """ renvoit la dernière config enregistree pour les sauvegardes sur les
            supports distants
        """
        if isfile(FILENAMES['distant']):
            lines = file(FILENAMES['distant'], 'r').readlines()
            if len(lines)>0:
                datas = lines[0]
            else:
                return []
            return self.parse_distant_conf(datas)
        return []

    def _get_usb_conf(self):
        """ renvoit la dernière config enregistree pour les sauvegardes sur les
            supports usb
        """
        if isfile(FILENAMES['usb']):
            lines = file(FILENAMES['usb'], 'r').readlines()
            if len(lines)>0:
                datas = lines[0]
            else:
                return[]
            return self.parse_usb_conf(datas)
        return []

    def _get_actual_log_conf(self):
        """ renvoit  les paramètres de messagerie de bacula """
        code, retour = tools.command_statusoutput('%sbaculamessage.py -l' % BACULA_DIR)
        if ';' in retour:
            retour = retour.strip()
            datas = retour.split(';')
            smtp = datas[0]
            mail = datas[1]
            mailok = datas[2]
            return mail, mailok, smtp
        return '', '', ''

    def _get_validate_btn(self):
        """ renvoit la description du bouton valider pour le formulaire form_name """
        return M.Submit(href=ajax.valid(self.server_nb,
                                        self.name,
                                        ['config1', 'config2', 'config3'],
                                        container='return_msg_container'),
                        libelle="OK")

    def parse_distant_conf(self, datas):
        """ parse un fichier de conf distant """
        log.msg("Distant %s" % datas)
        firstparse = datas.split()
        machine = firstparse[0].split('/')[2]
        partage = '/'.join(firstparse[0].split('/')[3:])
        ipmachine = firstparse[1]
        if len(firstparse)==4:
            login = firstparse[2]
            password = firstparse[3]
        else:
            login = ''
            password = ''
        return [machine, ipmachine, partage, login, password]

    def parse_usb_conf(self, datas):
        """ parse un fichier de conf usb """
        path = datas.strip()
        return [path]

    def modify_conf(self):
        """ valide les modifications de configuration de bacula """
        sup = get_actual_support()
        message = ''
        if not self.form_result.has_key('config1'):
            raise Exception, "Erreur: il manque des données pour la configuration de bacula."
        self.save_type = self.form_result['config1'][0]['value'] # recuperation du type de support
        if sup != self.save_type:
            self._set_support(self.save_type)
            message += "La sauvegarde a été définie en %s.\\n" % self.save_type

        if self.save_type in ('distant', 'usb'):
            if not self.form_result.has_key('config2'):
                raise Exception, "Erreur: il manque des données pour la configuration de bacula."

        if self.save_type == 'distant': # modification apportees pour 'distant'
            datas = self.form_result['config2']
            changes = tools.format_form_result(datas)
            for key, value in changes.items():
                if value == '':
                    changes.pop(key)
            self.write_distant_conf(changes)
            message += "La configuration de sauvegarde de type distante a été effectuée."

        elif self.save_type == 'usb': # modification apportees pour 'usb'
            datas = tools.format_form_result(self.form_result['config2'])
            path = datas['usb_periph']
            self.write_usb_conf(path)
            message += "La configuration de sauvegarde de type usb a été effectuée."

        log_datas = tools.format_form_result(self.form_result['config3']) # modification des parametres de log
        smtp, mail, mailok = (log_datas['smtp'], log_datas['mail'], log_datas['mailok'])

        act_mail, act_mailok, act_smtp = self._get_actual_log_conf()

        if ( mail != '' or mailok != '' ) and smtp != '':
            if mail:
                status, response = test_mail(mail)
            if mailok:
                status, response = test_mail(mailok)
            if not status:
                message += response
            else:
                cmd = '%sbaculamessage.py -s %s' % (BACULA_DIR, smtp)
                if mail:
                    cmd += ' -m %s' % mail
                if mailok:
                    cmd += ' -o %s' % mailok
                code, retour = tools.command_statusoutput('%s' % cmd)
                if code:
                    message += "\\nErreur lors de la configuration de la messagerie."
                else:
                    message += "\\nLa messagerie a bien été configurée."
        elif mail != act_mail or mailok != act_mailok or smtp != act_smtp:
            code, retour = tools.command_statusoutput('%sbaculamessage.py -d' % (BACULA_DIR))
            if code :
                message += "\\nErreur à la suppression du mail."
            else:
                message += "\\nAucune adresse mail configurée."
        return dict(message=message)

    def write_distant_conf(self, changes):
        """ modifie le fichier de configuration de sauvegarde pour les supports distants"""
        if len(changes.keys()) == 3:
            conf = "//%s/%s %s" % (changes['machine'].strip(),
                                   changes['partage'].strip(),
                                   changes['ipmachine'].strip())
        elif len(changes.keys()) == 5:
            conf = "//%s/%s %s %s %s" % (changes['machine'].strip(),
                                         changes['partage'].strip(),
                                         changes['ipmachine'].strip(),
                                         changes['login'].strip(),
                                         changes['password'].strip())
        else:
            raise Exception, "La configuration de la sauvegarde a échouée (manque de paramètres)."
        fic = file(FILENAMES['distant'], 'w')
        fic.write(conf)
        fic.close()

    def write_usb_conf(self, changes):
        """ modifie le fichier de configuration pour les supports usb """
        fic = file(FILENAMES['usb'], 'w')
        fic.write(changes)
        fic.close()

    def _set_support(self, sup):
        """ definit le support de sauvegarde au niveau de bacula """
        if sup in ['distant' , 'usb', 'bande']:
            code, retour = tools.command_statusoutput('%sbaculasupport.py -s %s' % (BACULA_DIR, sup))

