# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
###########################################################################
"""
Libraire de fonctions
pour l'EAD de Scribe
"""
from twisted.python import log
from os.path import isfile
from os import popen
from scribe import ldapconf
from scribe import eoleldap
from scribe import eoleuser
from scribe import eolegroup
from ead2.backend.actions import tools
from ead2.backend.config.filenames import printers_conf
from ead2.backend.actions.scribe.tool import validate

## outil de listing utilisateur

def get_quota_classe(ldapuser, username):
    """
    calcul de la chaine à afficher pour qualifier l'utilisateur
    """
    user_type = ldapuser._get_type(username)
    if user_type == 'eleve':
        return "élève de %s" % ldapuser._get_attr(username, 'Divcod')[0]
    elif user_type == 'enseignant':
        return "enseignant"
    elif user_type == 'administratif':
        return 'personnel administratif'
    else:
        return 'autre'

def filter_by_alpha(users, alpha):
    """ filtre une liste d'utilisateur sur la première lettre """
    result = []
    for user in users:
        if user.lower().startswith(alpha.lower()):
            result.append(user)
    return result

def user_filter(_type='', domain='', _class='', namepart='', alpha_key='', group=''):
    """ filtre les utilisateurs selon des parametres
        on cree le filtre ldap et on lance une recherche
        :alpha_key: première lettre du nom de l'utilisateur
        :_type: type d'utilisateur (pupil/teacher/administratif/responsable)
        :domain: domaine mail
        :group: groupe des utilisateurs
        :class: classe de l'utilisateur (pour les élèves à priori,
                marche aussi pour les profs admin de classe)
        :size: nombre d'utilisateurs par page
    """
    filtre = "(&"
    if _type:
        if _type == 'pupil':
            filtre += ldapconf.ELEVE_FILTER
        elif _type == 'teacher':
            filtre += ldapconf.PROF_FILTER
        elif _type == 'administratif':
            filtre += ldapconf.ADMINISTRATIF_FILTER
        elif _type == 'responsable':
            filtre += ldapconf.RESPONSABLE_FILTER
        elif _type == 'other':
            filtre += ldapconf.AUTRE_FILTER
        else:
            log.err("type d'utilisateur inconnu : %s" % _type)
            filtre += ldapconf.USER_FILTER
    else:
        filtre += ldapconf.USER_FILTER
    if domain == 'empty':
        filtre += '(!(mail=*@*))'
    elif domain == 'externe':
        filtre += '(mail=*@*)'
        for dom in ldapconf.MAIL_DOMAIN.values():
            filtre += '(!(mail=*@%s))' % dom
    elif domain:
        filtre += "(mail=*@%s)" % domain
    if _class:
        filtre += "(Divcod=%s)" % _class
    if namepart:
        filtre += "(sn=*%s*)" % namepart
    filtre += ")"
    ldapentry = eoleldap.LdapEntry()
    ldapentry.ldap_admin.connect()
    usrs = ldapentry.ldap_admin._search(filtre, ['uid'])
    users = [user[1]['uid'][0]for user in usrs]
    if group:
        gr_users = ldapentry._get_members(group)
        users = [user for user in users if user in gr_users]
    users.sort()
    if alpha_key:
        users = filter_by_alpha(users, alpha_key)
    ldapentry.ldap_admin.close()
    return users

def _get_alpha_links():
    """ renvoit la description des liens alphabetique pour un objet select"""
    alphas = [_get_select_default_value()]
    alphas.extend([{'name':chr(i).upper(), 'libelle':chr(i).upper()}for i in range(97, 123)])
    return alphas

def _get_select_default_value():
    """ renvoie l'option vide pour un objet select """
    return {'name':'', 'libelle':'', 'default':'ok'}

def sort_users(users):
    """ trie les nom des utilisateurs par ordre alphabétique
        prenom.nom par nom
    """
    _list = []
    for user in users:
        login = user.split('.')
        if len(login)>1:
            _list.append("%s.%s"%(login[1], login[0]))
        else:
            _list.append("%s"%login[0])
    _list.sort()
    users = []
    for user in _list:
        login = user.split('.')
        if len(login)>1:
            users.append("%s.%s"%(login[1], login[0]))
        else:
            users.append("%s"%login[0])
    return users

def sort_list(myset):
    """
    transforme un set en liste triée
    """
    res = list(myset)
    res.sort()
    return res

## quotas

def _get_quotas(_type='depasse'):
    """ renvoie les quotas des utilisateurs du domaine
        dans le cas _type: qui prend les valeurs "proche", "depasse" ou "tous"
    """
    def _get_name_in_login(login):
        """ récupère le nom dans un login de type p.nom """
        login = login.split('.')
        if len(login)> 1 :
            return login[1]
        else:
            return login[0]

    def cmp_username(a, b):
        """ sort function to compare by username
        """
        ua = _get_name_in_login(a.get('username', '0.0'))
        ub = _get_name_in_login(b.get('username', '0.0'))
        if ua > ub:
            return 1
        elif ua == ub:
            return 0
        else:
            return -1

    cmd = "/usr/sbin/repquota -a"
    return_code, quota_lines = tools.command_statusoutput(cmd)
    if return_code:
        raise Exception, "Erreur : un problème s'est produit à la récupération des quotas"
    #FIXME: gestion de plusiseurs partitions avec quota
    quota_lines = quota_lines.splitlines()[5:]
    result = []
    ldapuser = eoleuser.User()
    ldapuser.ldap_admin.connect()
    for line in quota_lines:
        if line.startswith('***'):
            break
        user = line.split()
        if len(user) < 5:
            continue
        username = user[0]
        if username.startswith('#'):
            continue
        fullquota = float(user[3])
        quota = int(fullquota) / 1024
        if quota == 0:
            continue
        fullused = float(user[2])
        used = int(fullused) / 1024
        delai = user[5]
        if fullused > fullquota:
            classe = get_quota_classe(ldapuser, username)
            result.append({'username':username, 'used':used, 'quota':quota,
                           'delai':delai, 'color':'#fd8758', 'classe':classe})
        elif fullused+3072 > fullquota and _type in ['proche', 'tous']:
            classe = get_quota_classe(ldapuser, username)
            result.append({'username':username, 'used':used, 'quota':quota,
                           'color':'#fddc58', 'classe':classe})
        elif _type == 'tous':
            result.append({'username':username, 'used':used, 'quota':quota})
    result.sort(cmp_username)
    return result

## virus

def _get_viruses():
    """ renvoie la liste des virus détectés """
    file1 = "/var/log/syslog"
    file2 = "/var/log/syslog.1.gz"
    max_vir = 30
    mois = {}
    mois["jan"] = "janvier" #idem
    mois["feb"] = "février"
    mois["fév"] = "février"
    mois["mar"] = "mars" #idem
    mois["apr"] = "avril"
    mois["avr"] = "avril"
    mois["may"] = "mai"
    mois["mai"] = "mai"
    mois["jun"] = "juin"
    mois["jui"] = "juin"
    mois["jul"] = "juillet" #idem
    mois["aug"] = "août"
    mois["aou"] = "août"
    mois["sep"] = "septembre" #idem
    mois["oct"] = "octobre"
    mois["nov"] = "novembre"
    mois["dec"] = "décembre"
    result = []
    viruses = []
    if isfile(file1):
        fic = popen('grep "contains virus" '+file1)
        lines = fic.readlines()
        lines.reverse()
        result.extend(lines)
        fic.close()
    if isfile(file2):
        fic = popen('zgrep "contains virus" '+file2)
        lines = fic.readlines()
        lines.reverse()
        result.extend(lines)
        fic.close()
    if len(result) > max_vir:
        result = result[:max_vir]
    for ligne in result:
        try :
            moisfr = mois[ligne[0:3].lower()]
        except :
            moisfr = ligne[0:3]
        date = ligne[4:6]+" "+moisfr
        elts = ligne.split()
        fichier = ' '.join(elts[7:-3])
        virus = elts[-1][0:-1]
        client = 'unknown'
        viruses.append({'client':client, 'fichier':fichier, 'date':date, 'virus':virus})
    if len(result) == 0:
        return []
    return viruses

## historique de connexion

def _get_history():
    """ renvoie l'historique des connexions """
    cmd = "/usr/share/ead/historique.sh"
    return_code, result = tools.command_statusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à la récupération de l'historique des connexions."
    lines = [{'date':" ".join(line.split()[0:5]),
              'user':line.split()[5],
              'poste':"%s (%s)"%(line.split()[6], line.split()[8]),
              'system':line.split()[7]} for line in result.splitlines()]
    if len(lines) == 0:
        return [{'default':"Aucune entrée n'a été enregistrée."}]
    return lines

### gestion des imprimantes

def _parse_printers(printers_str):
    """ parse le résultat du script /user/share/ead/printers.sh
        les tests s'effectue sur les deux langues (an et fr)
        normalement dans l'ead2 c'est fr.
    """
    printers = []
    printer = {}
    for line in printers_str.splitlines():
        line = line.strip()
        if line.startswith("printer") or line.startswith("l'imprimante"):
            if printer != {}:
                printers.append(printer)
                printer = {}
            elt = line.split()
            printer_name = elt[1]
            printer['name'] = printer_name
            if "disabled" in line or "désactivée" in line:
                printer['stopped'] = 'ok'
            elif "idle" in line or "ne fait rien" in line:
                printer['stopped'] = "idle"
            else:
                printer['stopped'] = ""
        elif line.startswith('Description'):
            description = line[13:].strip()
            if "No Information Available" in description:
                printer['description'] = "Description : %s" % printer_name
            else:
                printer['description'] = line
        elif line.startswith('Location') or line.startswith('Lieu'):
            lieu = line.split(':')[1]
            if 'Location Unknown' in lieu:
                printer['lieu'] = "Emplacement non renseigné"
            else:
                printer['lieu'] = "Emplacement : %s" % lieu
    printers.append(printer)
    return printers

printer_script = printers_conf['script']
def _get_printers():
    """ renvoie la description des imprimantes disponibles """
    cmd = "%s all 2>&1" % printer_script
    return_code, printers_str = tools.command_statusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à la récupération de la liste des imprimantes."
    printers = _parse_printers(printers_str)
    return printers

def stop_printer(printer_name):
    """ stoppe une imprimante """
    cmd = "%s disable %s 2>&1" % (printer_script, printer_name)
    return_code, return_str = tools.command_statusoutput(cmd)
    if return_code:
        raise Exception, "Erreur à l'arrêt de l'imprimante %s, \\n %s" % (printer_name, return_str)
    # on controle que ça a marché
    printers = _get_printers()
    for printer in printers:
        if printer['name'] == printer_name:
            if printer['stopped'] != 'ok':
                return "Compte rendu de l'arrêt de %s \\n Erreur : vérifiez votre configuration." % printer_name
    return "Compte rendu de l'arrêt de %s \\n Réussite de l'arrêt de l'imprimante" % printer_name

def start_printer(printer_name):
    """ démarre une imprimante """
    cmd = "%s enable %s 2>&1" % (printer_script, printer_name)
    return_code, return_str = tools.command_statusoutput(cmd)
    if return_code:
        raise Exception, "Erreur au redémarrage de l'imprimante %s, \\n %s" % (printer_name, return_str)
    # on controle que ça a marché
    printers = _get_printers()
    for printer in printers:
        if printer['name'] == printer_name:
            if printer['stopped'] == 'ok':
                return "Compte rendu du démarrage de %s. \\n Erreur : vérifiez votre configuration." % printer_name
    return "Compte rendu du démarrage de %s. \\n Réussite du démarrage de l'imprimante." % printer_name

def _get_printers_job(printer_name=None, completed=False):
    """ renvoie la liste des travaux pour printer_name
        :completed:True/False  <travaux terminé>/<travaux en cours>
    """
    jobs = []
    if completed:
        cmd = "%s completed" % printer_script
    else:
        cmd = "%s not-completed" % printer_script
    if printer_name:
        cmd += " %s 2>&1" % printer_name
    else:
        cmd += " 2>&1"
    return_code, return_str = tools.command_statusoutput(cmd)
    if return_code not in [256, 0]:
        raise Exception, "Erreur lors de la récupération des travaux sur %s." % printer_name
    for job in return_str.splitlines():
        job = job.split()
        user = job[1]
        printid = job[0]
        date = job[3] + ' ' + job[4] + ' ' + job[5] + ' ' + job[6]
        heure = job[7]
        jobs.append({'user':user, 'date':date,
                     'printid':printid, 'heure':heure})
    return jobs

def cancel_print(printid):
    """ annulation d'un travail d'impression """
    cmd = "%s cancel %s" % (printer_script, printid)
    return_code, return_str = tools.command_statusoutput(cmd)
    if return_code:
        raise Exception, "Erreur lors de l'annulation de %s" % printid
    return "L'impression %s a bien été annulée." % printid

## Gestion des utilisateurs

## outils pour la gestion des roles (prof_admin et prof)
def get_prof_class(login):
    """ renvoie les classes du prof login """
    classes = eolegroup.Group().get_classes()
    return filter_class_by_prof(login, classes)

def get_prof_class_list(login):
    """ renvoie la liste des classes d'un prof """
    cls = []
    for classe in get_prof_class(login).values():
        cls.extend(classe)
    return cls

def filter_class_by_prof(prof, classes):
    """ renvoie les classes du prof prof """
    groups = eoleuser.User().get_attr(prof, 'Divcod')
    for niveau, classe_list in classes.items():
        nivo = []
        for classe in classe_list:
            if classe in groups:
                nivo.append(classe)
        if nivo != []:
            classes[niveau] = nivo
        else:
            classes.pop(niveau)
    return classes

def is_eleve_from_prof(eleve, prof):
    """ l'eleve est il membre d'une classe administrée par prof ?"""
    ldapuser = eoleuser.User()
    ldapuser.ldap_admin.connect()
    if ldapuser._get_attr(eleve, 'typeadmin'):
        #c'est un prof
        ldapuser.ldap_admin.close()
        return False
    cls = get_prof_class_list(prof)
    for classe in cls:
        if eleve in ldapuser._get_members(classe):
            ldapuser.ldap_admin.close()
            return True
    ldapuser.ldap_admin.close()
    return False

def _valid_mail(mailtype, mailperso):
    """
    fabrication de l'adresse mail
    en fonction des infos du formulaire
    """
    if mailtype == 'perso':
        return validate.test_mailperso(mailperso)
    else:
        return 'local'

# gestion des rôles sur scribe
def is_admin(roles):
    """
        Renvoie True si l'utilisateur est admin
    """
    return ('admin_pedago' in roles) or ('admin' in roles)

def is_prof_admin(roles):
    """
        L'utilisateur est prof_admin ?
    """
    return (not is_admin(roles)) and ('prof_admin' in roles)

def is_prof(roles):
    """
        L'utilisateur est prof ?
    """
    return (not is_prof_admin(roles)) and ('prof' in roles) \
            and (not is_admin(roles))

def is_administratif(roles):
    """
        L'utilisateur est administratif_scribe ?
    """
    return (not is_admin(roles)) and ('administratif_scribe' in roles)

def get_role(roles):
    """
        Renvoie un des trois rôles prof/admin/prof_admin
    """
    if is_admin(roles):
        return 'admin'
    elif is_prof_admin(roles):
        return 'prof_admin'
    else:
        return 'prof'
