# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action générique
#
#
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import Action, Dict
from ead2.backend.actions import tools
from ead2.backend.actions.seshat import seshat_tools as t
from ead2.config.config import dico
from seshat.backend import EAD_ROUTES, gen_routes

used_templates = ['main', 'form', 'formselect', 'accordion']

class Routes(Action):
    """ Page """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = "routes" #celui de perm_nomdumodule.ini
    libelle = "Routes Exim"
    category = "Messagerie"
    description = "Gestion des routes pour Exim"

    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                        keys=['server', 'action', 'delroute'])#server et action sont les clés systématiques

    form_result = Dict(default={}, doc='retour de formulaire', keys=['addroute'])

    def execute(self):
        params, self.server_nb = tools.get_request(self.request)
        # Dans le cas d'un formulaire:
        if self.form_result.has_key('addroute'):
            try:
                resultat = tools.format_form_result(self.form_result['addroute'])
                # datas: dico utilisé pour la templatisation
                domaine, adresse = resultat['domaine'], resultat['adresse']
                log.msg("+ ajoute d'une route %s %s" % (domaine, adresse))
                res = t.add_route(EAD_ROUTES, domaine, adresse)
                if res:
                    gen_routes()

            except Exception, message:
                #si il y a une erreur soulevée par nos soins, on renvoie le message
                log.err()
            self.form_result = {}
        elif params.has_key('delroute'):
            domaine, adresse = params['delroute'][0].split('#')
            res = t.del_route(EAD_ROUTES, domaine, adresse)
            if res:
                gen_routes()

        result = {} # variable renvoyée utilisée dans le template associé
        result.update(self._get_list(t.get_routes(EAD_ROUTES)))
        result.update(self._get_form())
        return self.send_all(result, template='seshat_routes',
                            templates=used_templates)

    def _get_list(self, routes):
        routes_list = []
        for dom, add in routes:
            libelle = "%s -> %s" % (dom, add)
            suppr_btn = dict(href=tools.make_js_link(self.server_nb, self.name,
                                                     delroute='%s#%s' % (dom, add)),
                             libelle="", title="", icone="/image/supprimer.gif", _class='btn')
            routes_list.append((libelle, suppr_btn))
        return dict(routes=routes_list)

    def _get_form(self):
        domaine = {'libelle': "Domaine établissement<br/><i>(exemple : etab.%s)</i>" % dico['domaine_messagerie_academie'],
                    'name':'domaine', 'inline':'ok', 'onblur':"testVoid('domaine', 'error_domaine');"}
        adresse = {'libelle':'Adresse ip associée',
                    'name':'adresse', 'inline':'ok', 'onblur':"testVoid('adresse', 'error_adresse');"}
        valid_btn = {'libelle':'Valider', 'title':'', 'icone':'/image/ok.gif',
                     'href':tools.make_form_link(self.server_nb, self.name, True, ['addroute'])}
        return dict(domaine=domaine, adresse=adresse, valid_btn=valid_btn)


    def valid(self, resultat):
        # on traite le resultat (execution d'une action ...)
        print resultat
        #...
        #if not ...:
        #    raise Exception , "Mon message d'erreur qui sera affiché"
        # on pourrait renvoyer des données plutôt qu'un message
        return dict(message="Le message de retour")
