# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# outils pour Bacula
#
###########################################################################

""" Outils permettant de gérer les sauvegardes bacula """
from os.path import isfile

from ead2.config.config import dico as dico_eole
from ead2.backend.config.config import BACULA_DIR, module
from ead2.backend.actions.tools import command_statusoutput
from ead2.backend.config.filenames import bacula as baculafiles
from ead2.backend.actions.lib.widgets import form as F

# templates communs
USED_TEMPLATES = ['main', 'formselect', 'form']

# fichiers de définition des supports bacula
FILENAMES = {'usb':baculafiles['usb'],
             'distant':baculafiles['distant'],
             'bande':'/etc/bacula/bacula-sd.conf'}

def get_actual_support():
    """ renvoie le type de support actuel """
    code, retour = command_statusoutput('%sbaculasupport.py -l' % BACULA_DIR)
    if retour in ['distant' , 'usb', 'bande']:
        return retour
    else:
        return ''

def get_bacula_form(pagename):
    """
        renvoie le formulaire adequat
    """
    form = dict(typesav=_typesave(),
                level=_level())
    if pagename in ('weekly', 'daily'):
        form['begin'] = _begin()
        form['hour'] = _hour()
    if pagename == 'daily':
        form['end'] = _end()
    return form

def _typesave():
    """
        Renvoie la balise select pour le choix de type de sauvegarde
    """
    # type de sauvegarde
    typesav = F.Select(name='typesav',
                       libelle="Programmer une sauvegarde",
                       inline=True)
    typesav.add_option('Complete',
                       libelle='Complete',
                       default=True)
    if module == 'horus':
        typesav.add_option('Restreinte',
                           libelle='Restreinte')
    return typesav

def _level():
    """
        Renvoie la balise pour la sélection du niveau de mise à jour
    """
    # niveau de mise à jour
    level = F.Select(name='niveau', libelle="de type", inline=True)
    level.add_option('Full', libelle='Totale')
    level.add_option('Incremental', libelle='Incrémentale')
    return level

def _begin():
    """
        Choix du jour de début
    """
    # choix du jour
    begin = F.Select(name='jourdebut', libelle='le', inline=True)
    for day, libelle in get_days():
        begin.add_option(day, libelle=libelle, group='jour de la semaine')
    begin.default('tue', group='jour de la semaine')
    return begin

def _end():
    """
        Choix du jour de fin
    """
    end = F.Select(name='jourfin', libelle='au', inline=True)
    for day, libelle in get_days():
        end.add_option(day, libelle=libelle, group='jour de la semaine')
    end.default('sat', group='jour de la semaine')
    return end

def _hour():
    """
        Choix de l'heure
    """
    hour = F.Select(name='heure', libelle='à', inline=True)
    for i in range(24):
        hour.add_option(str(i),
                        libelle="%d heure" % i,
                        group='heure de la journée')
    hour.default('0')
    return hour

def get_days(_day=None):
    """
        renvoie un dico jouranglais:jourfrancais
        accompagné par un tuple décrivant l'ordre
        :day: jour en anglais (dans ce cas ne renvoie que le jour
                                                     en français)
    """
    days = (('mon', 'lundi'), ('tue', 'mardi'),
            ('wed', 'mercredi'), ('thu', 'jeudi'),
            ('fri', 'vendredi'), ('sat', 'samedi'),
            ('sun', 'dimanche'))
    if _day:
        for day, jour in days:
            if day == _day:
                return day, jour
    else:
        return days

def bacula_active():
    """
        Teste si bacula est activé
    """
    return ('oui', 'oui', 'oui') ==  (dico_eole['active_bacula_fd'],
                                      dico_eole['active_bacula_sd'],
                                      dico_eole['active_bacula_dir'])

def bacula_configured():
    """
        Teste si bacula est configuré
    """
    sup = get_actual_support()
    return sup in FILENAMES.keys() and isfile(FILENAMES[sup])

def get_typesav(typesav=None):
    """ renvoie un dico décrivant les type de sauvegarde
        ou la traduction de l'appelation système"""
    dico = {'Complete': 'Complète', 'Restreinte': 'Restreinte'}
    if typesav != None:
        return dico[typesav]
    return dico

def get_niveau(niveau=None):
    """ renvoie un dico : {'niveau': 'libelle'}
        ou la traduction de l'appelation système"""
    dico = {'Level=Incremental':'Incrémentale', 'Level=Full':'Totale' }
    dico2 = {'Incremental':'Incrémentale', 'Full':'Totale'}
    if niveau in dico.keys():
        return dico[niveau]
    elif niveau in dico2.keys():
        return dico2[niveau]
    else:
        return dico
