# -*- coding: UTF-8 -*-
"""
    Chargement des actions
"""
import imp
from os import listdir
from os.path import isfile, join, isdir

from twisted.python import log

from ead2.backend.lib import error

from ead2.backend.config.config import (CONFIG_DIR, ENABLED_ACTION_DIR,
                                        ACTION_DIR, module)

def _list_actions_from_file():
    """
        Liste les actions disponibles depuis des fichiers textes
    """
    # lecture des actions à charger
    fic_actions = ['actions.cfg']
    if isfile(join(CONFIG_DIR, 'actions_%s.cfg' % module)):
        fic_actions.append('actions_%s.cfg' % module)
    if isfile(join(CONFIG_DIR, 'actions_local.cfg')):
        fic_actions.append('actions_local.cfg')
    # liste des actions pour ce serveur
    for fic_action in fic_actions:
        list_f = file(join(CONFIG_DIR, fic_action))
        for action in list_f.readlines():
            if not action.startswith('#'):
                if isfile(join(ACTION_DIR,"%s.py" % (action.strip(),))):
                    yield action.strip()

def _list_auto_registered_actions(directory=ENABLED_ACTION_DIR):
    """
        Liste les actions auto-enregistrées
    """
    if isdir(directory):
        for filename in listdir(directory):
            if filename.endswith('.py') and filename != '__init__.py':
                yield filename[:-3]

def load_action_pyfile(action_file, action_dir):
    """
        Importe un fichier .py
    """
    try:
        log.msg(" * Chargement de \"%s.py\"" % join(action_dir, action_file))
        filename, pathname, desc = imp.find_module(action_file,
                                                  [action_dir])
        imp.load_module(action_file, filename, pathname, desc)
    except ImportError:
        log.err()
        log.msg("Error while loading module : \"%s\"" % (action_file))
    except Exception, msg:
        error.get_out("Error while loading the action file \"%s\": %s" % (
                                                            action_file, msg))


def load_actions():
    """
        charge les actions depuis le répertoire ACTION_DIR
    """
    log.msg("*** Chargement des actions")
    # chargement des actions enregistrés par fichier texte
    for action_file in _list_actions_from_file():
        load_action_pyfile(action_file, ACTION_DIR)
    # chargement des actions auto enregistrées
    log.msg("*** Chargement des actions auto")
    for action_file in _list_auto_registered_actions():
        load_action_pyfile(action_file, ENABLED_ACTION_DIR)
